@@ -239,7 +239,7 @@ template<typename T>
 class weak_atomic
 {
 public:
-	AE_NO_TSAN weak_atomic() { }
+	AE_NO_TSAN weak_atomic() : value() { }
 #ifdef AE_VCPP
 #pragma warning(push)
 #pragma warning(disable: 4100)		// Get rid of (erroneous) 'unreferenced formal parameter' warning
@@ -389,7 +389,7 @@ namespace moodycamel
 		    Semaphore& operator=(const Semaphore& other);
 
 		public:
-		    AE_NO_TSAN Semaphore(int initialCount = 0)
+		    AE_NO_TSAN Semaphore(int initialCount = 0) : m_hSema()
 		    {
 		        assert(initialCount >= 0);
 		        const long maxLong = 0x7fffffff;
@@ -437,7 +437,7 @@ namespace moodycamel
 		    Semaphore& operator=(const Semaphore& other);
 
 		public:
-		    AE_NO_TSAN Semaphore(int initialCount = 0)
+		    AE_NO_TSAN Semaphore(int initialCount = 0) : m_sema()
 		    {
 		        assert(initialCount >= 0);
 		        kern_return_t rc = semaphore_create(mach_task_self(), &m_sema, SYNC_POLICY_FIFO, initialCount);
@@ -460,11 +460,11 @@ namespace moodycamel
 				return timed_wait(0);
 			}
 
-			bool timed_wait(std::int64_t timeout_usecs) AE_NO_TSAN
+			bool timed_wait(std::uint64_t timeout_usecs) AE_NO_TSAN
 			{
 				mach_timespec_t ts;
 				ts.tv_sec = static_cast<unsigned int>(timeout_usecs / 1000000);
-				ts.tv_nsec = (timeout_usecs % 1000000) * 1000;
+				ts.tv_nsec = static_cast<int>((timeout_usecs % 1000000) * 1000);
 
 				// added in OSX 10.10: https://developer.apple.com/library/prerelease/mac/documentation/General/Reference/APIDiffsMacOSX10_10SeedDiff/modules/Darwin.html
 				kern_return_t rc = semaphore_timedwait(m_sema, ts);
@@ -497,10 +497,10 @@ namespace moodycamel
 		    Semaphore& operator=(const Semaphore& other);
 
 		public:
-		    AE_NO_TSAN Semaphore(int initialCount = 0)
+		    AE_NO_TSAN Semaphore(int initialCount = 0) : m_sema()
 		    {
 		        assert(initialCount >= 0);
-		        int rc = sem_init(&m_sema, 0, initialCount);
+		        int rc = sem_init(&m_sema, 0, static_cast<unsigned int>(initialCount));
 		        assert(rc == 0);
 		        AE_UNUSED(rc);
 		    }
@@ -537,8 +537,8 @@ namespace moodycamel
 				const int usecs_in_1_sec = 1000000;
 				const int nsecs_in_1_sec = 1000000000;
 				clock_gettime(CLOCK_REALTIME, &ts);
-				ts.tv_sec += (time_t)(usecs / usecs_in_1_sec);
-				ts.tv_nsec += (long)(usecs % usecs_in_1_sec) * 1000;
+				ts.tv_sec += static_cast<time_t>(usecs / usecs_in_1_sec);
+				ts.tv_nsec += static_cast<long>(usecs % usecs_in_1_sec) * 1000;
 				// sem_timedwait bombs if you have more than 1e9 in tv_nsec
 				// so we have to clean things up before passing it in
 				if (ts.tv_nsec >= nsecs_in_1_sec) {
@@ -606,7 +606,7 @@ namespace moodycamel
 					if (m_sema.wait())
 						return true;
 				}
-				if (timeout_usecs > 0 && m_sema.timed_wait(timeout_usecs))
+				if (timeout_usecs > 0 && m_sema.timed_wait(static_cast<uint64_t>(timeout_usecs)))
 					return true;
 				// At this point, we've timed out waiting for the semaphore, but the
 				// count is still decremented indicating we may still be waiting on
@@ -626,7 +626,7 @@ namespace moodycamel
 		    }
 
 		public:
-		    AE_NO_TSAN LightweightSemaphore(ssize_t initialCount = 0) : m_count(initialCount)
+		    AE_NO_TSAN LightweightSemaphore(ssize_t initialCount = 0) : m_count(initialCount), m_sema()
 		    {
 		        assert(initialCount >= 0);
 		    }
@@ -662,10 +662,10 @@ namespace moodycamel
 		        }
 		    }
 		    
-		    ssize_t availableApprox() const AE_NO_TSAN
+		    std::size_t availableApprox() const AE_NO_TSAN
 		    {
 		    	ssize_t count = m_count.load();
-		    	return count > 0 ? count : 0;
+		    	return count > 0 ? static_cast<std::size_t>(count) : 0;
 		    }
 		};
 	}	// end namespace spsc_sema
