@@ -10,6 +10,7 @@
 #include "minitest.h"
 #include "../common/simplethread.h"
 #include "../../readerwriterqueue.h"
+#include "../../readerwritercircularbuffer.h"
 
 using namespace moodycamel;
 
@@ -19,7 +20,18 @@ struct Foo
 {
 	Foo() : copied(false) { id = _id()++; }
 	Foo(Foo const& other) : id(other.id), copied(true) { }
-	~Foo()
+	Foo(Foo&& other) : id(other.id), copied(other.copied) { other.copied = true; }
+	Foo& operator=(Foo&& other)
+	{
+		verify();
+		id = other.id, copied = other.copied;
+		other.copied = true;
+		return *this;
+	}
+	~Foo() { verify(); }
+
+private:
+	void verify()
 	{
 		if (copied) return;
 		if (id != _last_destroyed_id() + 1) {
@@ -28,6 +40,8 @@ struct Foo
 		_last_destroyed_id() = id;
 		++_destroy_count();
 	}
+
+public:
 	static void reset() { _destroy_count() = 0; _id() = 0; _destroyed_in_order() = true; _last_destroyed_id() = -1; }
 	static int destroy_count() { return _destroy_count(); }
 	static bool destroyed_in_order() { return _destroyed_in_order(); }
@@ -95,6 +109,7 @@ public:
 		REGISTER_TEST(try_enqueue_fail_workaround);
 		REGISTER_TEST(try_emplace_fail);
 #endif
+		REGISTER_TEST(blocking_circular_buffer);
 	}
 	
 	bool create_empty_queue()
@@ -166,8 +181,6 @@ public:
 	
 	bool nonempty_destroy()
 	{
-		Foo item;
-		
 		// Some elements at beginning
 		Foo::reset();
 		{
@@ -175,6 +188,7 @@ public:
 			for (int i = 0; i != 10; ++i) {
 				q.enqueue(Foo());
 			}
+			ASSERT_OR_FAIL(Foo::destroy_count() == 0);
 		}
 		ASSERT_OR_FAIL(Foo::destroy_count() == 10);
 		ASSERT_OR_FAIL(Foo::destroyed_in_order());
@@ -186,6 +200,7 @@ public:
 			for (int i = 0; i != 31; ++i) {
 				q.enqueue(Foo());
 			}
+			ASSERT_OR_FAIL(Foo::destroy_count() == 0);
 		}
 		ASSERT_OR_FAIL(Foo::destroy_count() == 31);
 		ASSERT_OR_FAIL(Foo::destroyed_in_order());
@@ -197,6 +212,7 @@ public:
 			for (int i = 0; i != 94; ++i) {
 				q.enqueue(Foo());
 			}
+			ASSERT_OR_FAIL(Foo::destroy_count() == 0);
 		}
 		ASSERT_OR_FAIL(Foo::destroy_count() == 94);
 		ASSERT_OR_FAIL(Foo::destroyed_in_order());
@@ -205,20 +221,24 @@ public:
 		Foo::reset();
 		{
 			ReaderWriterQueue<Foo> q(31);
+			Foo item;
 			for (int i = 0; i != 42; ++i) {
 				q.enqueue(Foo());
 			}
+			ASSERT_OR_FAIL(Foo::destroy_count() == 0);
 			for (int i = 0; i != 31; ++i) {
 				ASSERT_OR_FAIL(q.try_dequeue(item));
 			}
+			ASSERT_OR_FAIL(Foo::destroy_count() == 31);
 		}
-		ASSERT_OR_FAIL(Foo::destroy_count() == 42);
+		ASSERT_OR_FAIL(Foo::destroy_count() == 43);
 		ASSERT_OR_FAIL(Foo::destroyed_in_order());
 		
 		// Some elements in multiple blocks
 		Foo::reset();
 		{
 			ReaderWriterQueue<Foo> q(31);
+			Foo item;
 			for (int i = 0; i != 123; ++i) {
 				q.enqueue(Foo());
 			}
@@ -241,7 +261,7 @@ public:
 				q.enqueue(Foo());
 			}
 		}
-		ASSERT_OR_FAIL(Foo::destroy_count() == 500);
+		ASSERT_OR_FAIL(Foo::destroy_count() == 501);
 		ASSERT_OR_FAIL(Foo::destroyed_in_order());
 		
 		return true;
@@ -434,7 +454,7 @@ public:
 					fence(memory_order_release);
 					front = front.load() + 1;
 				}
-				int size = (int)q.size_approx();
+				int size = static_cast<int>(q.size_approx());
 				fence(memory_order_acquire);
 				int tail_ = tail.load();
 				int front_ = front.load();
@@ -451,7 +471,7 @@ public:
 				int tail_ = tail.load();
 				int front_ = front.load();
 				fence(memory_order_acquire);
-				int size = (int)q.size_approx();
+				int size = static_cast<int>(q.size_approx());
 				if (size > tail_ - front_ || size < 0) {
 					result = 0;
 				}
@@ -469,7 +489,7 @@ public:
 		{
 			// this math for queue size estimation is only valid for q_size <= 256
 			for (size_t q_size = 2; q_size < 256; ++q_size) {
-				ReaderWriterQueue<int> q(q_size);
+				ReaderWriterQueue<size_t> q(q_size);
 				ASSERT_OR_FAIL(q.max_capacity() == ceilToPow2(q_size+1)-1);
 
 				const size_t start_cap = q.max_capacity();
@@ -675,6 +695,120 @@ public:
 		return true;
 	}
 #endif
+
+	bool blocking_circular_buffer()
+	{
+		{
+			// Basic enqueue
+			BlockingReaderWriterCircularBuffer<int> q(65);
+			for (int iteration = 0; iteration != 128; ++iteration) {  // check there's no problem with mismatch between nominal and allocated capacity
+				ASSERT_OR_FAIL(q.max_capacity() == 65);
+				ASSERT_OR_FAIL(q.size_approx() == 0);
+				ASSERT_OR_FAIL(q.try_enqueue(0));
+				ASSERT_OR_FAIL(q.max_capacity() == 65);
+				ASSERT_OR_FAIL(q.size_approx() == 1);
+				for (int i = 1; i != 65; ++i)
+					q.wait_enqueue(i);
+				ASSERT_OR_FAIL(q.size_approx() == 65);
+				ASSERT_OR_FAIL(!q.try_enqueue(65));
+
+				// Basic dequeue
+				int item;
+				ASSERT_OR_FAIL(q.try_dequeue(item));
+				ASSERT_OR_FAIL(item == 0);
+				for (int i = 1; i != 65; ++i) {
+					q.wait_dequeue(item);
+					ASSERT_OR_FAIL(item == i);
+				}
+				ASSERT_OR_FAIL(!q.try_dequeue(item));
+				ASSERT_OR_FAIL(!q.wait_dequeue_timed(item, 1));
+				ASSERT_OR_FAIL(item == 64);
+			}
+		}
+
+		{
+			// Zero capacity
+			BlockingReaderWriterCircularBuffer<int> q(0);
+			ASSERT_OR_FAIL(q.max_capacity() == 0);
+			ASSERT_OR_FAIL(!q.try_enqueue(1));
+			ASSERT_OR_FAIL(!q.wait_enqueue_timed(1, 0));
+		}
+
+		// Element lifetimes
+		Foo::reset();
+		{
+			BlockingReaderWriterCircularBuffer<Foo> q(31);
+			{
+				Foo item;
+				for (int i = 0; i != 23 + 32; ++i) {
+					ASSERT_OR_FAIL(q.try_enqueue(Foo()));
+					ASSERT_OR_FAIL(q.try_dequeue(item));
+				}
+				ASSERT_OR_FAIL(Foo::destroy_count() == 23 + 32);
+				ASSERT_OR_FAIL(Foo::destroyed_in_order());
+			}
+			Foo::reset();
+
+			{
+				Foo item;
+				for (int i = 0; i != 10; ++i)
+					ASSERT_OR_FAIL(q.try_enqueue(Foo()));
+				ASSERT_OR_FAIL(q.size_approx() == 10);
+				ASSERT_OR_FAIL(Foo::destroy_count() == 0);
+				ASSERT_OR_FAIL(q.try_dequeue(item));
+				ASSERT_OR_FAIL(q.size_approx() == 9);
+				ASSERT_OR_FAIL(Foo::destroy_count() == 1);
+			}
+			ASSERT_OR_FAIL(Foo::destroy_count() == 2);
+			ASSERT_OR_FAIL(Foo::destroyed_in_order());
+
+			BlockingReaderWriterCircularBuffer<Foo> q2(std::move(q));
+			ASSERT_OR_FAIL(q.size_approx() == 0);
+			ASSERT_OR_FAIL(q2.size_approx() == 9);
+
+			BlockingReaderWriterCircularBuffer<Foo> q3(2);
+			q3 = std::move(q2);
+			ASSERT_OR_FAIL(q.size_approx() == 0);
+			ASSERT_OR_FAIL(q2.size_approx() == 0);
+			ASSERT_OR_FAIL(q3.size_approx() == 9);
+
+			q = std::move(q2);
+			ASSERT_OR_FAIL(q.size_approx() == 0);
+			ASSERT_OR_FAIL(q2.size_approx() == 0);
+			ASSERT_OR_FAIL(q3.size_approx() == 9);
+			ASSERT_OR_FAIL(Foo::destroy_count() == 2);
+		}
+		ASSERT_OR_FAIL(Foo::destroy_count() == 11);
+		ASSERT_OR_FAIL(Foo::destroyed_in_order());
+
+		weak_atomic<int> result;
+		result = 1;
+
+		{
+			// Threaded
+			BlockingReaderWriterCircularBuffer<int> q(8);
+			SimpleThread reader([&]() {
+				int item;
+				for (int i = 0; i != 1000000; ++i) {
+					q.wait_dequeue(item);
+					if (item != i)
+						result = 0;
+				}
+			});
+			SimpleThread writer([&]() {
+				for (int i = 0; i != 1000000; ++i)
+					q.wait_enqueue(i);
+			});
+
+			writer.join();
+			reader.join();
+
+			ASSERT_OR_FAIL(q.size_approx() == 0);
+			ASSERT_OR_FAIL(result.load());
+		}
+
+		return true;
+	}
 };
 
 
